/*
 *  GPSdo - Software for GPSDO
 *  (c) Ad Sekeris 2019
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "configuration.h"
#ifdef LANG_GB
  #include "language_gb.h"
#elif LANG_NL
  #include "language_nl.h"
#endif

#include <TinyGPS++.h>

#ifdef LCD_2004 or LCD_1206
  #include <LiquidCrystal_I2C.h>
  //#include <LcdBarGraphX.h>
#endif
#ifdef Si5351_present
  #include "si5351.h"   // Bibliotheek 'Etherkit Si5351' 
#endif
#include <ctype.h>

#ifdef LCD_2004
  LiquidCrystal_I2C lcd(LCD_I2C_addr,20,4);  // set the LCD address for a 20 chars and 4 line display
  #if LCD_PHASE_IND 
    //LcdBarGraphX lbg(&lcd, 20, 0, 3);
  #endif
#elif LCD_1602
  LiquidCrystal_I2C lcd(LCD_I2C_addr,16,2);  // set the LCD address for a 16 chars and 2 line display
  #if LCD_PHASE_IND 
    //LcdBarGraphX lbg(&lcd, 16, 0, 1);
  #endif
#endif
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif

 // Prototype definition for functions in external files
void configureUblox(byte *settingsArrayPointer);
void LcdmeterInit();
void LCD2004Meter();

bool Fresh_Start = true;
TinyGPSPlus gps;
TinyGPSCustom satsInView(gps, "GPGSV", 3);         // $GPGSV sentence, third element
TinyGPSCustom satFixMode(gps, "GPGSA", 2);         // $GPGSA sentence, third element
#ifdef Si5351_present
  Si5351 si5351;
#endif

char GPS_SATS[] = "  /  ";
String PLL_STAT;
char UTC_TIME[13];
char UTC_DATE[15];
int SatsInUse;
int GpsFixStat;

void setup()
{
  analogReference(INTERNAL1V1);
  //analogReference(INTERNAL2V56);
  //analogReference(DEFAULT);
  // START The USB SERIAL PORT
  Serial.begin(9600);
  // START The GPS SERIAL PORT
  Serial1.begin(9600);
  #ifdef LCD_2004 or LCD_1602
    lcd.init();                       // initialize the lcd
    lcd.backlight();                  // Lcd backlight on
    #if LCD_PHASE_IND 
      LcdmeterInit();                   // Setup special chars for the phase indicator
    #endif    
    #ifdef LCD_2004
      lcd.setCursor(3, 1);
    #elif LCD_1602
      lcd.setCursor(0, 0);
    #endif  
    lcd.print(MSG_WELCOME);
    #ifdef LCD_2004
      lcd.setCursor(2, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif  
    lcd.print(MSG_SETUP);
  #endif
  configureUblox(settingsArray);
  Serial1.end();
  lcd.clear();
  if (!gpsStatus[0]) {
    #ifdef LCD_2004
      lcd.setCursor(5, 1);
    #elif LCD_1602
      lcd.setCursor(3, 0);
    #endif  
    lcd.print(MSG_GPS);
    #ifdef LCD_2004
      lcd.setCursor(5, 2);
    #elif LCD_1602
      lcd.setCursor(3, 1);
    #endif  
    lcd.print(MSG_FAIL);
    while (true) {
      // do nothing forever
    }
  }
  Serial1.begin(9600);
  lcd.clear();
  #ifdef LCD_2004
    lcd.setCursor(3, 1);
  #elif LCD_1602
    lcd.setCursor(0, 0);
  #endif  
  lcd.print(MSG_WAIT_FIX);

#ifdef Si5351_present
  // See configuration.h for the startup values. Default is everything off
  si5351.init(SI5351_CRYSTAL_LOAD_0PF, Si5351_reference, 0);
  si5351.drive_strength(SI5351_CLK0, Si5351_Clk0_level);
  si5351.set_freq(Si5351_Clk0_freq, SI5351_CLK0);
  si5351.output_enable(SI5351_CLK0, Si5351_Clk0_status);
  si5351.drive_strength(SI5351_CLK1, Si5351_Clk1_level);
  si5351.set_freq(Si5351_Clk1_freq, SI5351_CLK1);
  si5351.output_enable(SI5351_CLK1, Si5351_Clk1_status);
  si5351.drive_strength(SI5351_CLK2, Si5351_Clk2_level);
  si5351.set_freq(Si5351_Clk2_freq, SI5351_CLK2);
  si5351.output_enable(SI5351_CLK2, Si5351_Clk2_status);
#endif
}

void loop()
{
  bool newData = false;
  unsigned long chars;
  unsigned short sentences, failed;
  int val = 0;  // variable to store the value read


  if(Serial1.available())
  {
    char c = Serial1.read();  
    // Wite every character to GPS for decoding.
    if (gps.encode(c)) // Did a new valid sentence come in?
      newData = true;
    // THIS IS THE MAIN LOOP JUST READS IN FROM THE GPS SERIAL AND ECHOS OUT TO THE ARDUINO SERIAL.
    //Serial.write(c);
    if (!USB_DEBUG) Serial.write(c);
  }
  if (newData)
  {
    if (Fresh_Start) {
      lcd.clear();
      Fresh_Start = false;
    }
    if (satFixMode.isUpdated()){
      GpsFixStat = String(satFixMode.value()).toInt();
      if (GpsFixStat == 3) PLL_STAT = MSG_GPS_LOCK_3D;
      else if (GpsFixStat == 2) String PLL_STAT = MSG_GPS_LOCK_2D;
      else PLL_STAT = MSG_GPS_UNLOCK;
      #ifdef LCD_2004
        lcd.setCursor(0, 0);
      #elif LCD_1602
        lcd.setCursor(0, 0);
      #endif  
      lcd.print(PLL_STAT);
    }
    if (satsInView.isUpdated()){
      SatsInUse = String(gps.satellites.value()).toInt();
      sprintf(GPS_SATS,"%2d/%2d", SatsInUse, String(satsInView.value()).toInt());
      #ifdef LCD_2004
        lcd.setCursor(15, 0);
      #elif LCD_1602
        lcd.setCursor(11, 0);
      #endif  
      lcd.print(GPS_SATS);
    }
    if (gps.time.isUpdated()){
      #ifdef LCD_2004
        lcd.setCursor(0, 1);
      #elif LCD_1602
        lcd.setCursor(0, 1);
      #endif
      FillUtcTime();
      lcd.print(UTC_TIME);    
  //val = analogRead(A0);  // read the input pin
  //Serial.println(val);          // debug value
    }
    if (gps.date.isUpdated()){
      #ifdef LCD_2004
        lcd.setCursor(0, 2);
      #elif LCD_1602
        lcd.setCursor(0, 2);
      #endif
      FillUtcDate();
      lcd.print(UTC_DATE);    
    }

      #ifdef LCD_2004
        #if LCD_PHASE_IND 
          LCD2004Meter();
        #endif
      #endif
      //float flat, flon;
    //unsigned long age;
    //gps.f_get_position(&flat, &flon, &age);
    //Serial.print("LAT=");
    //Serial.print(flat == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flat, 6);
    //lcd.clear();
    //lcd.setCursor(0, 0);
    //lcd.print("LAT: "); 
    //lcd.print(flat == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flat, 6);
    //Serial.print(" LON=");
    //Serial.print(flon == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flon, 6);
    //lcd.setCursor(0, 1);
    //lcd.print("LON: ");
    //lcd.print(flon == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flon, 6);
    //Serial.print(" SAT=");
    //Serial.print(gps.satellites() == TinyGPS::GPS_INVALID_SATELLITES ? 0 : gps.satellites());
    //Serial.print(" PREC=");
    //Serial.println(gps.hdop() == TinyGPS::GPS_INVALID_HDOP ? 0 : gps.hdop());
    newData = false;
  }

}
