/*
 *  GPSdo - Software for GPSDO
 *  (c) Ad Sekeris 2019
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: configuration.h - contains all hardware settings
 */

  // Language selector - only one should be true
  #define LANG_GB true
  #define LANG_NL false

  // Select debug on off -
  // off - All NMEA messages are printed on the USB serial
  // on  - Program debug messages are printed on the USB serial
  #define USB_DEBUG false

  // USB serial NMEA otput settings
  #define USB_NMEA false // true or false
  #define USB_BAUD 9600 // Valid baudrate for the reciving application. 8 data and 1 stop bit. no parity.
  
  // Display settings.
  #define LCD_I2C_addr 0x27  // bus address of the I2C lcd display
  #define OLED_I2C_addr 0x78  // bus address of the I2C oled display
  // Select only one of the following items true.
  #define LCD_2004 true
  #define LCD_1602 false
  #define OLED_128x64 false
  #define NEXTION_35 false
  
  #define NEXTION_size 35  // Size of the nextion display
  // Phase indicator on LCD
  #define LCD_PHASE_IND false // Software not functional. Do not change!!

// RS232_C settings
  #define RS232C_present false  // hardware is mounted on the board
  #define RS232C_baud    9600  // Startup baudrate (8 databits, 1 stop, no parity)  

// Si5351A print GPSDO4
  #define Si5351_present true  // hardware is mounted on the board = true
  #define Si5351_addr 0x60  // bus address of the I2C Si5351 chip
  #define Si5351_reference 10000000 // 10MHz reference input for the chip
  // Startup values after boot.
  // Uitgangs vermogen per kanaal zonder filtering
  // SI5351_DRIVE_2MA = 2mA in 50 Ohm ong. -1dBm
  // SI5351_DRIVE_4MA = 4mA in 50 Ohm ong. 5dBm
  // SI5351_DRIVE_6MA = 6mA in 50 Ohm ong. 7dBm
  // SI5351_DRIVE_8MA = 8mA in 50 Ohm ong. 8dBm
  #define Si5351_Clk0_status 1 // 0 = off - 1 = on
  #define Si5351_Clk0_freq 2500000000ULL // 25MHz start frequency for LNB reference
  #define Si5351_Clk0_level SI5351_DRIVE_2MA // Set output level
  #define Si5351_Clk1_status 1 // 0 = off - 1 = on
  #define Si5351_Clk1_freq 1000000000ULL // 10MHz start frequency
  #define Si5351_Clk1_level SI5351_DRIVE_2MA // Set output level
  #define Si5351_Clk2_status 1 // 0 = off - 1 = on
  #define Si5351_Clk2_freq 4000000000ULL // 15.6MHz start frequency
  #define Si5351_Clk2_level SI5351_DRIVE_2MA // Set output level

// Variables for UBlox-7M setup
  boolean gpsStatus[] = {false, false, false, false, false, false, false, false, false};
  //unsigned long start;
  //
  //Settings Array contains the following settings: [0]NavMode, [1]DataRate1, [2]DataRate2, [3]PortRateByte1, [4]PortRateByte2, [5]PortRateByte3,
  //[6]NMEA GLL Sentence, [7]NMEA GSA Sentence, [8]NMEA GSV Sentence, [9]NMEA RMC Sentence, [10]NMEA VTG Sentence  {11} Powermode
  //NavMode: [0]
  //Stationary Mode    = 0x02
  //Pedestrian Mode    = 0x03
  //Automotive Mode    = 0x04
  //Sea Mode           = 0x05
  //Airborne < 1G Mode = 0x06
  //
  //DataRate: [1] [2]
  //1Hz     = 0xE8 0x03
  //2Hz     = 0xF4 0x01
  //3.33Hz  = 0x2C 0x01
  //4Hz     = 0xFA 0x00
  //
  //PortRate: [3] [4] [5]
  //4800   = C0 12 00
  //9600   = 80 25 00
  //19200  = 00 4B 00  **SOFTWARESERIAL LIMIT FOR ARDUINO UNO R3!**
  //38400  = 00 96 00  **SOFTWARESERIAL LIMIT FOR ARDUINO MEGA 2560!**
  //57600  = 00 E1 00
  //115200 = 00 C2 01
  //230400 = 00 84 03
  //
  //NMEA Messages: [6] [7] [8] [9] [10]
  //OFF = 0x00
  //ON  = 0x01
  //
  //RXM power managenent: [11]
  //0x00 = Continous mode
  //0x01 = Power save mode
  //
  byte settingsArray[] = {0x02, 0xE8, 0x03, 0x80, 0x25, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00}; //
