/*
 *  GPSdo - Software for GPSDO
 *  (c) Ad Sekeris 2019
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: ublox.ino - contains all gps setup related software
 */

void configureUblox(byte *settingsArrayPointer) {
  byte gpsSetSuccess = 0;
  char GPAsuccess[] = "         ";
  //Configuring u-Blox GPS initial state...
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif  
    lcd.print(GPAsuccess);
  #endif
  //Generate the configuration string for Navigation Mode NAV5
  byte setNav[] = {0xB5, 0x62, 0x06, 0x24, 0x24, 0x00, 0xFF, 0xFF, *settingsArrayPointer, 0x03, 0x00, 0x00, 0x00, 0x00, 0x10, 0x27, 0x00, 0x00, 0x05, 0x00, 0xFA, 0x00, 0xFA, 0x00, 0x64, 0x00, 0x2C, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setNav[2], sizeof(setNav) - 4);

  //Generate the configuration string for Data Rate 
  byte setDataRate[] = {0xB5, 0x62, 0x06, 0x08, 0x06, 0x00, settingsArrayPointer[1], settingsArrayPointer[2], 0x01, 0x00, 0x01, 0x00, 0x00, 0x00};
  calcChecksum(&setDataRate[2], sizeof(setDataRate) - 4);

  //Generate the configuration string for Baud Rate
  byte setPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, settingsArrayPointer[3], settingsArrayPointer[4], settingsArrayPointer[5], 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setPortRate[2], sizeof(setPortRate) - 4);

  // GLL messages on or off
  byte setGLL[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x01, settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], 0x01, 0x00, 0x00};
  calcChecksum(&setGLL[2], sizeof(setGLL) - 4);
  
  // GSA messages on or off
  byte setGSA[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x02, settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], 0x01, 0x00, 0x00};
  calcChecksum(&setGSA[2], sizeof(setGSA) - 4);
  
  // GSV messages on or off
  byte setGSV[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x03, settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], 0x01, 0x00, 0x00};
  calcChecksum(&setGSV[2], sizeof(setGSV) - 4);
  
  // RMC messages on or off
  byte setRMC[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x04, settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], 0x01, 0x00, 0x00};
  calcChecksum(&setRMC[2], sizeof(setRMC) - 4);
  
  // VTG messages on or off
  byte setVTG[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x05, settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], 0x00, 0x00, 0x00};
  calcChecksum(&setVTG[2], sizeof(setVTG) - 4);
  
  // Power management
  byte setPWRMG[] = {0xB5, 0x62, 0x06, 0x11, 0x02, 0x00, 0x08, settingsArrayPointer[11], 0x00, 0x00};
  calcChecksum(&setPWRMG[2], sizeof(setPWRMG) - 4);
  // Timepulse TP5 1Hz = geen sat 10MHz is fix
  byte setTP5[] = {0xB5, 0x62, 0x06, 0x31, 0x20, 0x00, 0x00, 0x01, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x96, 0x98, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00,
                   0x00, 0x00, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setTP5[2], sizeof(setTP5) - 4);

  delay(500);

  while(gpsSetSuccess < 3)
  {
    //Setting Navigation Mode...
    sendUBX(&setNav[0], sizeof(setNav));  //Send UBX Packet
    gpsSetSuccess += getUBX_ACK(&setNav[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 5) {
      gpsSetSuccess -= 4;
      setBaud(settingsArrayPointer[4]);
      delay(1500);
      byte lowerPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x80, 0x25, 0x00, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA2, 0xB5};
      sendUBX(lowerPortRate, sizeof(lowerPortRate));
      Serial1.begin(9600);
      delay(2000);
    }
    if(gpsSetSuccess == 6) gpsSetSuccess -= 4;
    if (gpsSetSuccess == 10) gpsStatus[0] = true;
  }
  GPAsuccess[0] = '-';
  if (gpsSetSuccess == 0xa) {
    //Navigation mode configuration succes.
    GPAsuccess[0] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);

  while(gpsSetSuccess < 3) {
    //Setting Data Update Rate...
    sendUBX(&setDataRate[0], sizeof(setDataRate));  //Send UBX Packet
    gpsSetSuccess += getUBX_ACK(&setDataRate[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 10) gpsStatus[1] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[1] = '-';
  if (gpsSetSuccess == 0xa) {
    //Data Update Rate configuration succes.
    GPAsuccess[1] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);

  while(gpsSetSuccess < 3) {
    //Setup for NMEA GLL Messages
    sendUBX(setGLL, sizeof(setGLL));
    gpsSetSuccess += getUBX_ACK(&setGLL[2]);
    if (gpsSetSuccess == 10) gpsStatus[2] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[2] = '-';
  if (gpsSetSuccess == 0xa) {
    //Setup for NMEA GLL succes.
    GPAsuccess[2] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);
 
  while(gpsSetSuccess < 3) {
    //Setup for NMEA GSA Messages
    sendUBX(setGSA, sizeof(setGSA));
    gpsSetSuccess += getUBX_ACK(&setGSA[2]);
    if (gpsSetSuccess == 10) gpsStatus[3] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 3) {
    //NMEA GSA Message setup Failed!
    sendUBX(setGLL, sizeof(setGLL));
    gpsSetSuccess += getUBX_ACK(&setGLL[2]);
    if (gpsSetSuccess == 10) gpsStatus[2] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[3] = '-';
  if (gpsSetSuccess == 0xa) {
    //Setup for NMEA GSA succes.
    GPAsuccess[3] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);
  
  while(gpsSetSuccess < 3) {
    //Setup for NMEA GSV Messages
    sendUBX(setGSV, sizeof(setGSV));
    gpsSetSuccess += getUBX_ACK(&setGSV[2]);
    if (gpsSetSuccess == 10) gpsStatus[4] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[4] = '-';
  if (gpsSetSuccess == 0xa) {
    //Setup for NMEA GSV succes.
    GPAsuccess[4] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);
  
  while(gpsSetSuccess < 3) {
    //Setup for NMEA RMC Messages
    sendUBX(setRMC, sizeof(setRMC));
    gpsSetSuccess += getUBX_ACK(&setRMC[2]);
    if (gpsSetSuccess == 10) gpsStatus[5] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[5] = '-';
  if (gpsSetSuccess == 0xa) {
    //Setup for NMEA RMC succes.
    GPAsuccess[5] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);
  
  while(gpsSetSuccess < 3) {
   //Setup for NMEA VTG Messages
    sendUBX(setVTG, sizeof(setVTG));
    gpsSetSuccess += getUBX_ACK(&setVTG[2]);
    if (gpsSetSuccess == 10) gpsStatus[6] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[6] = '-';
  if (gpsSetSuccess == 0xa) {
    //Setup for NMEA VTG succes.
    GPAsuccess[6] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);

  while(gpsSetSuccess < 3 && settingsArrayPointer[11] == 0x00) {
    //Set power mode
    sendUBX(setPWRMG, sizeof(setPWRMG));
    gpsSetSuccess += getUBX_ACK(&setPWRMG[2]);
    if (gpsSetSuccess == 10) gpsStatus[7] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[7] = '-';
  if (gpsSetSuccess == 0xa) {
    //Set power mode succes!
    GPAsuccess[7] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);

  while(gpsSetSuccess < 3 && settingsArrayPointer[11] == 0x00) {
    //Set time pulse output
    sendUBX(setTP5, sizeof(setTP5));
    gpsSetSuccess += getUBX_ACK(&setTP5[2]);
    if (gpsSetSuccess == 10) gpsStatus[8] = true;
    if (gpsSetSuccess == 5 | gpsSetSuccess == 6) gpsSetSuccess -= 4;
  }
  GPAsuccess[8] = '-';
  if (gpsSetSuccess == 0xa) {
    //Set time pulse output succes!
    GPAsuccess[8] = '+';
  }
  #ifdef LCD_2004 or LCD_1602
    #ifdef LCD_2004
      lcd.setCursor(9, 2);
    #elif LCD_1602
      lcd.setCursor(0, 1);
    #endif 
    lcd.print(GPAsuccess);
  #endif
  gpsSetSuccess = 0;
  delay(250);

//  if (settingsArrayPointer[4] != 0x25) {
//    Serial.print("Setting Port Baud Rate... ");
//    sendUBX(&setPortRate[0], sizeof(setPortRate));
//    setBaud(settingsArrayPointer[4]);
//    Serial.println("Success!");
//    delay(10000);
//  }
}


void calcChecksum(byte *checksumPayload, byte payloadSize) {
  byte CK_A = 0, CK_B = 0;
  for (int i = 0; i < payloadSize ;i++) {
    CK_A = CK_A + *checksumPayload;
    CK_B = CK_B + CK_A;
    checksumPayload++;
  }
  *checksumPayload = CK_A;
  checksumPayload++;
  *checksumPayload = CK_B;
}

void sendUBX(byte *UBXmsg, byte msgLength) {
  for(int i = 0; i < msgLength; i++) {
    Serial1.write(UBXmsg[i]);
    Serial1.flush();
  }
  Serial1.println();
  Serial1.flush();
}


byte getUBX_ACK(byte *msgID) {
  byte CK_A = 0, CK_B = 0;
  byte incoming_char;
  boolean headerReceived = false;
  unsigned long ackWait = millis();
  byte ackPacket[10] = {0xB5, 0x62, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  int i = 0;
  while (1) {
    if (Serial1.available()) {
      incoming_char = Serial1.read();
      if (incoming_char == ackPacket[i]) {
        i++;
      }
      else if (i > 2) {
        ackPacket[i] = incoming_char;
        i++;
      }
    }
    if (i > 9) break;
    if ((millis() - ackWait) > 1500) {
      if (USB_DEBUG) Serial.println("ACK Timeout");
      return 5;
    }
    if (i == 4 && ackPacket[3] == 0x00) {
      if (USB_DEBUG) Serial.println("NAK Received");
      return 1;
    }
  }

  for (i = 2; i < 8 ;i++) {
  CK_A = CK_A + ackPacket[i];
  CK_B = CK_B + CK_A;
  }
  if (msgID[0] == ackPacket[6] && msgID[1] == ackPacket[7] && CK_A == ackPacket[8] && CK_B == ackPacket[9]) {
    if (USB_DEBUG) Serial.println("Success!");
    if (USB_DEBUG) Serial.print("ACK Received! ");
    printHex(ackPacket, sizeof(ackPacket));
    return 10;
        }
  else {
    if (USB_DEBUG) Serial.print("ACK Checksum Failure: ");
    printHex(ackPacket, sizeof(ackPacket));
    delay(500);
    return 1;
  }
}


void printHex(uint8_t *data, uint8_t length) // prints 8-bit data in hex
{
  char tmp[length*2+1];
  byte first ;
  int j=0;
  for (byte i = 0; i < length; i++)
  {
    first = (data[i] >> 4) | 48;
    if (first > 57) tmp[j] = first + (byte)7;
    else tmp[j] = first ;
    j++;

    first = (data[i] & 0x0F) | 48;
    if (first > 57) tmp[j] = first + (byte)7;
    else tmp[j] = first;
    j++;
  }
  tmp[length*2] = 0;
  for (byte i = 0, j = 0; i < sizeof(tmp); i++) {
    if (USB_DEBUG) Serial.print(tmp[i]);
    if (j == 1) {
      if (USB_DEBUG) Serial.print(" ");
      j = 0;
    }
    else j++;
  }
  if (USB_DEBUG) Serial.println();
}

void setBaud(byte baudSetting) {
  if (baudSetting == 0x12) Serial1.begin(4800);
  if (baudSetting == 0x25) Serial1.begin(9600);
  if (baudSetting == 0x4B) Serial1.begin(19200);
  if (baudSetting == 0x96) Serial1.begin(38400);
  if (baudSetting == 0xE1) Serial1.begin(57600);
  if (baudSetting == 0xC2) Serial1.begin(115200);
  if (baudSetting == 0x84) Serial1.begin(230400);
}
